<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/**
 * CodeIgniter
 *
 * An open source application development framework for PHP 5.1.6 or newer
 *
 * @package     CI-GUIDE.INFO
 * @author      ci-guide.info Dev Team
 * @copyright   Copyright (c) 2012, www.ci-guide.info
 * @license     http://codeigniter.com/user_guide/license.html
 * @link        http://www.ci-guide.info
 * @since       Version 1.0
 * @filesource
 */

// ------------------------------------------------------------------------

/**
 * CodeIgniter-Guide Cig_openid Class
 *
 * OpenID Library
 *
 * @package     CI-GUIDE.INFO
 * @subpackage  Libraries
 * @category    Libraries
 * @author      ci-guide.info Dev Team
 * @link        http://www.ci-guide.info/hack/penid/
 */
class CIG_Openid {

    protected $CI;
    protected $openid_provider       = '';
    protected $openid_identifier     = '';
    protected $openid_form_id        = 'openid_message';
    protected $openid_realm          = '';
    protected $openid_return_to      = '';
    protected $openid_filestore_path = '/tmp/openid';
    protected $openid_assoc_type     = NULL;

    /**
     * Constructor
     */
    public function __construct($config = array())
    {
        $this->CI =& get_instance();
        $this->CI->load->library('session');
        $this->CI->config->load('cig_openid');

        if (! isset($config['php_openid_path']) || empty($config['php_openid_path'])) {
            $config['php_openid_path'] = $this->CI->config->item('php_openid_path');
        }

        if (! isset($config['openid_provider']) || empty($config['openid_provider'])) {
            $config['openid_provider'] = $this->CI->config->item('openid_provider');
        }

        if (isset($config['openid_provider']) && ! empty($config['openid_provider'])) {
            $config = array_merge($config, $this->CI->config->item($config['openid_provider']));
        }

        $include_path = get_include_path();
        set_include_path($include_path . ':' . $config['php_openid_path']);
        require_once 'Auth/OpenID/FileStore.php';
        require_once 'Auth/OpenID/Consumer.php';
        require_once dirname( __FILE__) . '/Cig_phpsession.php';


        $this->initialize($config);

    }

    // -------------------------------------------------------------------------------------

    /**
     *
     */
    public function initialize($config=array())
    {

        if (is_array($config) && count($config) > 0)
        {
            $keys = array(
                'openid_provider',
                'openid_identifier',
                'openid_form_id',
                'openid_realm',
                'openid_return_to',
                'openid_filestore_path',
                'openid_assoc_type',
                'openid_xrds',
            );

            foreach ($keys as $key)
            {
                if (isset($config[$key])) {
                    $this->$key = $config[$key];
                }
            }
        }

        $this->CI->load->helper('url');

        if (! strlen($this->openid_realm) > 0)
        {
            $this->openid_realm =site_url();
        }
        elseif (! preg_match('|^http|', $this->openid_realm))
        {
            $site_url = site_url();
            $this->openid_realm = rtrim($site_url, '/') . $this->openid_realm;
        }

        if (! strlen($this->openid_return_to) > 0)
        {
            $this->openid_return_to = site_url();
        }
        elseif (! preg_match('|^http|', $this->openid_return_to))
        {
            $site_url = site_url();
            $this->openid_return_to = rtrim($site_url, '/') . $this->openid_return_to;
        }

        if (isset($this->openid_xrds) && ! preg_match('|^http|', $this->openid_xrds)) {
            $site_url = site_url();
            $this->openid_xrds = rtrim($site_url, '/') . $this->openid_xrds;
        }

        log_message('debug', 'Cig_openid Class Initialized');

    }

    // -------------------------------------------------------------------------------------

    /**
     *
     */
    public function set_provider($provider)
    {
        if (is_string($provider) && strlen($provider) > 0)
        {
            $config = $this->CI->config->item($provider);

            if (is_array($config) && count($config > 0))
            {
                log_message('debug', 'Set OpenID Provider : ' . $provider);
                $this->openid_provider = $provider;
                $this->initialize($config);
                return TRUE;
            }
        }

        log_message('error', 'invalid OpenID Provider.');
        return FALSE;
    }

    // -------------------------------------------------------------------------------------

    /**
     * Try OpenID Authentication process
     *
     * @access  public
     * @return  string
     */
    public function try_auth()
    {
        $store = new Auth_OpenID_FileStore($this->openid_filestore_path);
        $consumer = new Auth_OpenID_Consumer($store, new Cig_PHPSession($this->CI->session));

        // Set Association Type
        if (isset($this->openid_assoc_type) && is_array($this->openid_assoc_type) && count($this->openid_assoc_type) > 0) {
            //$order[] = array('HMAC-SHA256', 'DH-SHA256');
            $order[] = $this->openid_assoc_type;
            $consumer->consumer->negotiator = new Auth_OpenID_SessionNegotiator($order);
        }

        // Begin the OpenID authentication process.
        if (! $request = $consumer->begin($this->openid_identifier)) {
            // No auth request means we can't begin OpenID.
            log_message('error', 'OpenID Begin Failed.');
            return FALSE;
        }

        // For OpenID 1, send a redirect.  For OpenID 2, use a Javascript
        // form to send a POST request to the server.
        if ($request->shouldSendRedirect())
        {
            $redirect_url = $request->redirectURL($this->openid_realm, $this->openid_return_to);
            if (Auth_OpenID::isFailure($redirect_url))
            {
                log_message('error', 'Auth_OpenID Failure.');
                return FALSE;
            }
            else
            {
                $this->CI->load->helper('url');
                redirect($redirect_url);
                return TRUE;
            }
        }
        else
        {

            $form_html = $request->htmlMarkup($this->openid_realm,
                                              $this->openid_return_to,
                                              FALSE,
                                              array('id' => $this->openid_form_id));

            if ($this->openid_provider === 'docomo_openid' || $this->openid_provider === 'ntt_openid') {
                //log_message('debug', "\n" . $form_html);
                $form_html = $this->_sanitize4docomo($form_html);
            }

            return $form_html;
        }
    }

    private function _sanitize4docomo($html)
    {
        return preg_replace_callback('|(<input type="hidden" name="openid.assoc_handle" value=")(.*)(" />)|', 
            create_function(
                '$matches',
                'return $matches[1].preg_replace("|\s|", "+", $matches[2]).$matches[3];'
            ),
            $html
        );
    }


    // -------------------------------------------------------------------------------------

    /**
     * Complete the authentication process
     *
     * @access  public
     * @return  string
     */
    public function finish_auth()
    {
        $store = new Auth_OpenID_FileStore($this->openid_filestore_path);
        $consumer = new Auth_OpenID_Consumer($store, new Cig_PHPSession($this->CI->session));

        $response = $consumer->complete($this->openid_return_to);

        // Check the response status.
        if ($response->status == Auth_OpenID_CANCEL)
        {
            // This means the authentication was cancelled.
            log_message('error', 'Auth_OpenID Finish CANCEL.');
            return FALSE;
        }
        elseif ($response->status == Auth_OpenID_FAILURE)
        {
            // Authentication failed
            log_message('error', 'Auth_OpenID Finish Failure.');
            return FALSE;
        }
        elseif ($response->status == Auth_OpenID_SUCCESS)
        {
            // This means the authentication succeeded
            $openid = $response->getDisplayIdentifier();
            return htmlentities($openid);
        }
        else
        {
            log_message('error', 'Auth_OpenID Finish Error.');
            return FALSE;
        }

    }

    // -------------------------------------------------------------------------------------

    /**
     * Set 'X-XRDS-Location' HTTP Header
     */
    public function set_xrds_location()
    {
        if (isset($this->openid_xrds)) {
            $this->CI->output->set_header('X-XRDS-Location: ' . $this->openid_xrds);
        }
    }

    // -------------------------------------------------------------------------------------

    /**
     * Build XRDS
     *
     * @access  public
     * @return  string
     */
    public function xrds($return_to=NULL, $output=TRUE)
    {
        if (is_null($return_to) || ! is_string($return_to)) {
            $return_to = $this->openid_return_to;
        }

        $xrds = '<?xml version="1.0" encoding="UTF-8"?>' . "\n"
              . '<xrds:XRDS' . "\n"
              . '  xmlns:xrds="xri://$xrds"' . "\n"
              . '  xmlns:openid="http://openid.net/xmlns/1.0"' . "\n"
              . '  xmlns="xri://$xrd*($v*2.0)">' . "\n"
              . '    <XRD>' . "\n"
              . '    <Service priority="0">' . "\n"
              . '       <Type>http://specs.openid.net/auth/2.0/return_to</Type>' . "\n"
              . '       <URI>' . $return_to . '</URI>' . "\n"
              . '    </Service>' . "\n"
              . '    </XRD>' . "\n"
              . '</xrds:XRDS>' . "\n";

        if ($output === FALSE)
        {
            return $xrds;
        }
        else
        {
            $this->CI->output->set_output($xrds);
        }
    }
}

// END Cig_openid class

/* End of file Cig_openid.php */
/* Location: ./application/third_party/ci-guide.info/libraries/Cig_openid.php */
